@@ -15,9 +15,8 @@ class CHyprXWaylandManager {
     SP<CWLSurfaceResource> getWindowSurface(PHLWINDOW);
     void                   activateSurface(SP<CWLSurfaceResource>, bool);
     void                   activateWindow(PHLWINDOW, bool);
-    void                   getGeometryForWindow(PHLWINDOW, CBox*);
+    CBox                   getGeometryForWindow(PHLWINDOW);
     void                   sendCloseWindow(PHLWINDOW);
-    void                   setWindowSize(PHLWINDOW, Vector2D, bool force = false);
     void                   setWindowFullscreen(PHLWINDOW, bool);
     bool                   shouldBeFloated(PHLWINDOW, bool pending = false);
     void                   checkBorders(PHLWINDOW);
@@ -25,4 +24,4 @@ class CHyprXWaylandManager {
     Vector2D               waylandToXWaylandCoords(const Vector2D&);
 };
 
-inline std::unique_ptr<CHyprXWaylandManager> g_pXWaylandManager;
\ No newline at end of file
+inline UP<CHyprXWaylandManager> g_pXWaylandManager;
\ No newline at end of file
