@@ -5,13 +5,13 @@
 class CHyprBorderDecoration : public IHyprWindowDecoration {
   public:
     CHyprBorderDecoration(PHLWINDOW);
-    virtual ~CHyprBorderDecoration();
+    virtual ~CHyprBorderDecoration() = default;
 
     virtual SDecorationPositioningInfo getPositioningInfo();
 
     virtual void                       onPositioningReply(const SDecorationPositioningReply& reply);
 
-    virtual void                       draw(CMonitor*, float a);
+    virtual void                       draw(PHLMONITOR, float const& a);
 
     virtual eDecorationType            getDecorationType();
 
